@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2016-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2016-2026, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -8,89 +8,105 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 import json
-import copy
+import dataclasses as dc
 from io import IOBase, TextIOBase
-from typing import Any, Dict, List, Optional, Type, Union, Tuple, \
-    IO, BinaryIO, TextIO, Iterator
+from collections.abc import Iterator
+from functools import partial
+from typing import Any, BinaryIO, IO, Optional, TextIO, Union
+from xml.etree import ElementTree
 
-from elementpath.etree import ElementTree, etree_tostring
+from xmlschema.exceptions import XMLSchemaTypeError, XMLSchemaValueError, XMLResourceError
+from xmlschema.names import XSD_NAMESPACE, XSI_TYPE, XSD_SCHEMA
+from xmlschema.aliases import ElementType, NsmapType, LocationsType, SourceArgType, \
+    DecodeType, EncodeType, JsonDecodeType, XMLSourceType, SchemaType
+from xmlschema.translation import gettext as _
+from xmlschema.arguments import Argument, validate_type, BooleanOption, ValidationOption
+from xmlschema.utils.etree import is_etree_document, etree_tostring
+from xmlschema.utils.qnames import get_extended_qname, update_namespaces, get_namespace_map
+from xmlschema.resources import fetch_schema_locations, XMLResource
+from xmlschema.converters import ConverterType
+from xmlschema.validators import XMLSchema10, XMLSchemaBase, XMLSchemaValidationError
+from xmlschema.arguments import LocationsOption
+from xmlschema.settings import ResourceSettings, SchemaSettings
 
-from .exceptions import XMLSchemaTypeError, XMLSchemaValueError, XMLResourceError
-from .names import XSD_NAMESPACE, XSI_TYPE, XSD_SCHEMA
-from .aliases import ElementType, XMLSourceType, NamespacesType, LocationsType, \
-    LazyType, SchemaSourceType, ConverterType, DecodeType, EncodeType, \
-    JsonDecodeType
-from .helpers import get_extended_qname, is_etree_document
-from .resources import fetch_schema_locations, XMLResource
-from .validators import XMLSchema10, XMLSchemaBase, XMLSchemaValidationError
+__all__ = ('from_json', 'is_valid', 'iter_errors', 'iter_decode', 'to_dict',
+           'to_etree', 'to_json', 'validate', 'XmlDocument')
+
+RESOURCE_KWARGS = frozenset(fld.name for fld in dc.fields(ResourceSettings))
+SCHEMA_KWARGS = frozenset(fld.name for fld in dc.fields(SchemaSettings))
+
+
+class SchemaArgument(Argument[SchemaType]):
+    _validators = partial(validate_type, types=XMLSchemaBase),
+
+    def __set__(self, instance: Any, value: Any) -> None:
+        setattr(instance, self._name, self.validated_value(value))
 
 
 def get_context(xml_document: Union[XMLSourceType, XMLResource],
-                schema: Optional[Union[XMLSchemaBase, SchemaSourceType]] = None,
-                cls: Optional[Type[XMLSchemaBase]] = None,
-                locations: Optional[LocationsType] = None,
-                base_url: Optional[str] = None,
-                defuse: str = 'remote',
-                timeout: int = 300,
-                lazy: LazyType = False,
-                use_location_hints: bool = True,
-                dummy_schema: bool = False) -> Tuple[XMLResource, XMLSchemaBase]:
+                schema: Optional[Union[XMLSchemaBase, SourceArgType]] = None,
+                cls: Optional[type[XMLSchemaBase]] = None,
+                **kwargs: Any) -> tuple[XMLResource, SchemaType]:
     """
     Get the XML document validation/decode context.
 
     :return: an XMLResource instance and a schema instance.
     """
     resource: XMLResource
-    kwargs: Dict[Any, Any]
+    if not isinstance(xml_document, XMLResource):
+        _kwargs = {k: kwargs[k] for k in kwargs if k in RESOURCE_KWARGS}
+        resource = XMLResource(xml_document, **_kwargs)
+    elif not isinstance(xml_document, XmlDocument):
+        resource = xml_document
+    else:
+        return xml_document, xml_document.schema
 
+    _kwargs = {k: kwargs[k] for k in kwargs if k in SCHEMA_KWARGS}
+    return resource, get_resource_schema(resource, schema, cls, **_kwargs)
+
+
+def get_resource_schema(resource: XMLResource,
+                        schema: Optional[Union[XMLSchemaBase, SourceArgType]] = None,
+                        cls: Optional[type[XMLSchemaBase]] = None,
+                        validation: str = 'strict',
+                        locations: Optional[LocationsType] = None,
+                        use_location_hints: bool = True,
+                        **kwargs: Any) -> SchemaType:
     if cls is None:
         cls = XMLSchema10
     elif not issubclass(cls, XMLSchemaBase):
-        raise XMLSchemaTypeError("invalid schema class %r" % cls)
-
-    if isinstance(xml_document, XMLResource):
-        resource = xml_document
-    else:
-        resource = XMLResource(xml_document, base_url, defuse=defuse,
-                               timeout=timeout, lazy=lazy)
+        raise XMLSchemaTypeError(_("invalid schema class {!r}").format(cls))
 
     if isinstance(schema, XMLSchemaBase) and resource.namespace in schema.maps.namespaces:
-        return resource, schema
-    if isinstance(resource, XmlDocument) and isinstance(resource.schema, XMLSchemaBase):
-        return resource, resource.schema
+        return schema
 
     if use_location_hints:
         try:
-            schema_location, locations = fetch_schema_locations(
-                resource, locations, base_url=base_url
-            )
+            schema_location, locations = fetch_schema_locations(resource, locations, **kwargs)
         except ValueError:
             pass
         else:
-            kwargs = dict(locations=locations, defuse=defuse, timeout=timeout)
+            kwargs['locations'] = locations
             if schema is None or isinstance(schema, XMLSchemaBase):
-                return resource, cls(schema_location, **kwargs)
+                return cls(schema_location, **kwargs)
             else:
-                return resource, cls(schema, **kwargs)
+                return cls(schema, **kwargs)
 
-    if schema is None:
-        if cls.meta_schema is not None and \
-                (XSI_TYPE in resource.root.attrib or XSD_NAMESPACE in resource.namespace):
-            return resource, cls.meta_schema
-        elif dummy_schema:
-            return resource, get_dummy_schema(resource.root.tag, cls)
-        else:
-            msg = "cannot get a schema for XML data, provide a schema argument"
-            raise XMLSchemaValueError(msg)
-
-    elif isinstance(schema, XMLSchemaBase):
-        return resource, schema
+    if isinstance(schema, XMLSchemaBase):
+        return schema  # fallback to a schema for a different namespace
+    elif schema is not None:
+        return cls(schema, locations=locations, **kwargs)
+    elif XSD_NAMESPACE == resource.namespace:
+        assert cls.meta_schema is not None
+        return cls.meta_schema
+    elif validation == 'skip' or XSI_TYPE in resource.root.attrib:
+        return get_dummy_schema(resource.root.tag, cls)
     else:
-        return resource, cls(schema, locations=locations, base_url=base_url,
-                             defuse=defuse, timeout=timeout)
+        msg = _("cannot get a schema for XML data, provide a schema argument")
+        raise XMLSchemaValueError(msg)
 
 
-def get_dummy_schema(tag: str, cls: Type[XMLSchemaBase]) -> XMLSchemaBase:
+def get_dummy_schema(tag: str, cls: type[XMLSchemaBase]) -> XMLSchemaBase:
     if tag.startswith('{'):
         namespace, name = tag[1:].split('}')
     else:
@@ -98,19 +114,19 @@ def get_dummy_schema(tag: str, cls: Type[XMLSchemaBase]) -> XMLSchemaBase:
 
     if namespace:
         return cls(
-            '<xs:schema xmlns:xs="{0}" targetNamespace="{1}">\n'
-            '    <xs:element name="{2}"/>\n'
+            '<xs:schema xmlns:xs="{}" targetNamespace="{}">\n'
+            '    <xs:element name="{}"/>\n'
             '</xs:schema>'.format(XSD_NAMESPACE, namespace, name)
         )
     else:
         return cls(
-            '<xs:schema xmlns:xs="{0}">\n'
-            '    <xs:element name="{1}"/>\n'
+            '<xs:schema xmlns:xs="{}">\n'
+            '    <xs:element name="{}"/>\n'
             '</xs:schema>'.format(XSD_NAMESPACE, name)
         )
 
 
-def get_lazy_json_encoder(errors: List[XMLSchemaValidationError]) -> Type[json.JSONEncoder]:
+def get_lazy_json_encoder(errors: list[XMLSchemaValidationError]) -> type[json.JSONEncoder]:
 
     class JSONLazyEncoder(json.JSONEncoder):
         def default(self, obj: Any) -> Any:
@@ -128,17 +144,14 @@ def get_lazy_json_encoder(errors: List[XMLSchemaValidationError]) -> Type[json.J
 
 def validate(xml_document: Union[XMLSourceType, XMLResource],
              schema: Optional[XMLSchemaBase] = None,
-             cls: Optional[Type[XMLSchemaBase]] = None,
+             cls: Optional[type[XMLSchemaBase]] = None,
              path: Optional[str] = None,
              schema_path: Optional[str] = None,
              use_defaults: bool = True,
-             namespaces: Optional[NamespacesType] = None,
+             namespaces: Optional[NsmapType] = None,
              locations: Optional[LocationsType] = None,
-             base_url: Optional[str] = None,
-             defuse: str = 'remote',
-             timeout: int = 300,
-             lazy: LazyType = False,
-             use_location_hints: bool = True) -> None:
+             use_location_hints: bool = True,
+             **kwargs: Any) -> None:
     """
     Validates an XML document against a schema instance. This function builds an
     :class:`XMLSchema` object for validating the XML document. Raises an
@@ -162,76 +175,64 @@ def validate(xml_document: Union[XMLSourceType, XMLResource],
     :param namespaces: is an optional mapping from namespace prefix to URI.
     :param locations: additional schema location hints, used if a schema instance \
     has to be built.
-    :param base_url: is an optional custom base URL for remapping relative locations, for \
-    default uses the directory where the XSD or alternatively the XML document is located.
-    :param defuse: optional argument to pass for construct schema and \
-    :class:`XMLResource` instances.
-    :param timeout: optional argument to pass for construct schema and \
-    :class:`XMLResource` instances.
-    :param lazy: optional argument for construct the :class:`XMLResource` instance.
     :param use_location_hints: for default, in case a schema instance has \
     to be built, uses also schema locations hints provided within XML data. \
-    Set this option to `False` to ignore these schema location hints.
+    set this option to `False` to ignore these schema location hints.
+    :param kwargs: other optional arguments for building :class:`XMLResource` or \
+    :class:`XMLSchema` instances provided as keyword arguments.
     """
-    source, _schema = get_context(xml_document, schema, cls, locations, base_url,
-                                  defuse, timeout, lazy, use_location_hints)
-    _schema.validate(source, path, schema_path, use_defaults, namespaces)
+    kwargs.update(locations=locations, use_location_hints=use_location_hints)
+    source, schema = get_context(xml_document, schema, cls, **kwargs)
+    schema.validate(source, path, schema_path, use_defaults, namespaces,
+                    use_location_hints=use_location_hints)
 
 
 def is_valid(xml_document: Union[XMLSourceType, XMLResource],
              schema: Optional[XMLSchemaBase] = None,
-             cls: Optional[Type[XMLSchemaBase]] = None,
+             cls: Optional[type[XMLSchemaBase]] = None,
              path: Optional[str] = None,
              schema_path: Optional[str] = None,
              use_defaults: bool = True,
-             namespaces: Optional[NamespacesType] = None,
+             namespaces: Optional[NsmapType] = None,
              locations: Optional[LocationsType] = None,
-             base_url: Optional[str] = None,
-             defuse: str = 'remote',
-             timeout: int = 300,
-             lazy: LazyType = False,
-             use_location_hints: bool = True) -> bool:
+             use_location_hints: bool = True,
+             **kwargs: Any) -> bool:
     """
     Like :meth:`validate` except that do not raise an exception but returns ``True`` if
     the XML document is valid, ``False`` if it's invalid.
     """
-    source, schema = get_context(xml_document, schema, cls, locations, base_url,
-                                 defuse, timeout, lazy, use_location_hints)
-    return schema.is_valid(source, path, schema_path, use_defaults, namespaces)
+    kwargs.update(validation='lax', locations=locations, use_location_hints=use_location_hints)
+    source, schema = get_context(xml_document, schema, cls, **kwargs)
+    return schema.is_valid(source, path, schema_path, use_defaults, namespaces,
+                           use_location_hints=use_location_hints)
 
 
 def iter_errors(xml_document: Union[XMLSourceType, XMLResource],
                 schema: Optional[XMLSchemaBase] = None,
-                cls: Optional[Type[XMLSchemaBase]] = None,
+                cls: Optional[type[XMLSchemaBase]] = None,
                 path: Optional[str] = None,
                 schema_path: Optional[str] = None,
                 use_defaults: bool = True,
-                namespaces: Optional[NamespacesType] = None,
+                namespaces: Optional[NsmapType] = None,
                 locations: Optional[LocationsType] = None,
-                base_url: Optional[str] = None,
-                defuse: str = 'remote',
-                timeout: int = 300,
-                lazy: LazyType = False,
-                use_location_hints: bool = True) -> Iterator[XMLSchemaValidationError]:
+                use_location_hints: bool = True,
+                **kwargs: Any) -> Iterator[XMLSchemaValidationError]:
     """
     Creates an iterator for the errors generated by the validation of an XML document.
     Takes the same arguments of the function :meth:`validate`.
     """
-    source, schema = get_context(xml_document, schema, cls, locations, base_url,
-                                 defuse, timeout, lazy, use_location_hints)
-    return schema.iter_errors(source, path, schema_path, use_defaults, namespaces)
+    kwargs.update(validation='lax', locations=locations, use_location_hints=use_location_hints)
+    source, schema = get_context(xml_document, schema, cls, **kwargs)
+    return schema.iter_errors(source, path, schema_path, use_defaults, namespaces,
+                              use_location_hints=use_location_hints)
 
 
 def iter_decode(xml_document: Union[XMLSourceType, XMLResource],
                 schema: Optional[XMLSchemaBase] = None,
-                cls: Optional[Type[XMLSchemaBase]] = None,
+                cls: Optional[type[XMLSchemaBase]] = None,
                 path: Optional[str] = None,
                 validation: str = 'lax',
                 locations: Optional[LocationsType] = None,
-                base_url: Optional[str] = None,
-                defuse: str = 'remote',
-                timeout: int = 300,
-                lazy: LazyType = False,
                 use_location_hints: bool = True,
                 **kwargs: Any) -> Iterator[Union[Any, XMLSchemaValidationError]]:
     """
@@ -253,36 +254,30 @@ def iter_decode(xml_document: Union[XMLSourceType, XMLResource],
     'strict', 'lax' or 'skip'.
     :param locations: additional schema location hints, in case a schema instance \
     has to be built.
-    :param base_url: is an optional custom base URL for remapping relative locations, for \
-    default uses the directory where the XSD or alternatively the XML document is located.
-    :param defuse: optional argument to pass for construct schema and \
-    :class:`XMLResource` instances.
-    :param timeout: optional argument to pass for construct schema and \
-    :class:`XMLResource` instances.
-    :param lazy: optional argument for construct the :class:`XMLResource` instance.
     :param use_location_hints: for default, in case a schema instance has \
     to be built, uses also schema locations hints provided within XML data. \
-    Set this option to `False` to ignore these schema location hints.
+    set this option to `False` to ignore these schema location hints.
     :param kwargs: other optional arguments of :meth:`XMLSchemaBase.iter_decode` \
+    or for building :class:`XMLResource` or :class:`XMLSchema` instances provided \
     as keyword arguments.
     :raises: :exc:`XMLSchemaValidationError` if the XML document is invalid and \
     ``validation='strict'`` is provided.
     """
-    source, _schema = get_context(xml_document, schema, cls, locations, base_url,
-                                  defuse, timeout, lazy, use_location_hints)
-    yield from _schema.iter_decode(source, path=path, validation=validation, **kwargs)
+    kwargs.update(
+        validation=validation,
+        locations=locations,
+        use_location_hints=use_location_hints
+    )
+    source, _schema = get_context(xml_document, schema, cls, **kwargs)
+    yield from _schema.iter_decode(source, path=path, **kwargs)
 
 
 def to_dict(xml_document: Union[XMLSourceType, XMLResource],
             schema: Optional[XMLSchemaBase] = None,
-            cls: Optional[Type[XMLSchemaBase]] = None,
+            cls: Optional[type[XMLSchemaBase]] = None,
             path: Optional[str] = None,
             validation: str = 'strict',
             locations: Optional[LocationsType] = None,
-            base_url: Optional[str] = None,
-            defuse: str = 'remote',
-            timeout: int = 300,
-            lazy: LazyType = False,
             use_location_hints: bool = True,
             **kwargs: Any) -> DecodeType[Any]:
     """
@@ -294,24 +289,24 @@ def to_dict(xml_document: Union[XMLSourceType, XMLResource],
     :raises: :exc:`XMLSchemaValidationError` if the XML document is invalid and \
     ``validation='strict'`` is provided.
     """
-    source, _schema = get_context(xml_document, schema, cls, locations, base_url,
-                                  defuse, timeout, lazy, use_location_hints)
-    return _schema.decode(source, path=path, validation=validation, **kwargs)
+    kwargs.update(
+        validation=validation,
+        locations=locations,
+        use_location_hints=use_location_hints
+    )
+    source, _schema = get_context(xml_document, schema, cls, **kwargs)
+    return _schema.decode(source, path=path, **kwargs)
 
 
 def to_json(xml_document: Union[XMLSourceType, XMLResource],
             fp: Optional[IO[str]] = None,
             schema: Optional[XMLSchemaBase] = None,
-            cls: Optional[Type[XMLSchemaBase]] = None,
+            cls: Optional[type[XMLSchemaBase]] = None,
             path: Optional[str] = None,
             validation: str = 'strict',
             locations: Optional[LocationsType] = None,
-            base_url: Optional[str] = None,
-            defuse: str = 'remote',
-            timeout: int = 300,
-            lazy: LazyType = False,
             use_location_hints: bool = True,
-            json_options: Optional[Dict[str, Any]] = None,
+            json_options: Optional[dict[str, Any]] = None,
             **kwargs: Any) -> JsonDecodeType:
     """
     Serialize an XML document to JSON. For default the XML data is validated during
@@ -333,16 +328,9 @@ def to_json(xml_document: Union[XMLSourceType, XMLResource],
     'strict', 'lax' or 'skip'.
     :param locations: additional schema location hints, in case the schema instance \
     has to be built.
-    :param base_url: is an optional custom base URL for remapping relative locations, for \
-    default uses the directory where the XSD or alternatively the XML document is located.
-    :param defuse: optional argument to pass for construct schema and \
-    :class:`XMLResource` instances.
-    :param timeout: optional argument to pass for construct schema and \
-    :class:`XMLResource` instances.
-    :param lazy: optional argument for construct the :class:`XMLResource` instance.
     :param use_location_hints: for default, in case a schema instance has \
     to be built, uses also schema locations hints provided within XML data. \
-    Set this option to `False` to ignore these schema location hints.
+    set this option to `False` to ignore these schema location hints.
     :param json_options: a dictionary with options for the JSON serializer.
     :param kwargs: optional arguments of :meth:`XMLSchemaBase.iter_decode` as keyword arguments \
     to variate the decoding process.
@@ -352,19 +340,24 @@ def to_json(xml_document: Union[XMLSourceType, XMLResource],
     :raises: :exc:`XMLSchemaValidationError` if the object is not decodable by \
     the XSD component, or also if it's invalid when ``validation='strict'`` is provided.
     """
-    source, _schema = get_context(xml_document, schema, cls, locations, base_url,
-                                  defuse, timeout, lazy, use_location_hints)
+    kwargs.update(
+        validation=validation,
+        locations=locations,
+        use_location_hints=use_location_hints
+    )
+    source, _schema = get_context(xml_document, schema, cls, **kwargs)
     if json_options is None:
         json_options = {}
     if 'decimal_type' not in kwargs:
         kwargs['decimal_type'] = float
 
-    errors: List[XMLSchemaValidationError] = []
+    errors: list[XMLSchemaValidationError] = []
 
     if path is None and source.is_lazy() and 'cls' not in json_options:
         json_options['cls'] = get_lazy_json_encoder(errors)
 
-    obj = _schema.decode(source, path=path, validation=validation, **kwargs)
+    obj = _schema.decode(source, path=path, **kwargs)
+
     if isinstance(obj, tuple):
         errors.extend(obj[1])
         if fp is not None:
@@ -382,11 +375,11 @@ def to_json(xml_document: Union[XMLSourceType, XMLResource],
 
 
 def to_etree(obj: Any,
-             schema: Optional[Union[XMLSchemaBase, SchemaSourceType]] = None,
-             cls: Optional[Type[XMLSchemaBase]] = None,
+             schema: Optional[Union[XMLSchemaBase, SourceArgType]] = None,
+             cls: Optional[type[XMLSchemaBase]] = None,
              path: Optional[str] = None,
              validation: str = 'strict',
-             namespaces: Optional[NamespacesType] = None,
+             namespaces: Optional[NsmapType] = None,
              use_defaults: bool = True,
              converter: Optional[ConverterType] = None,
              unordered: bool = False,
@@ -461,15 +454,15 @@ def to_etree(obj: Any,
 
 
 def from_json(source: Union[str, bytes, IO[str]],
-              schema: Optional[Union[XMLSchemaBase, SchemaSourceType]] = None,
-              cls: Optional[Type[XMLSchemaBase]] = None,
+              schema: Optional[Union[XMLSchemaBase, SourceArgType]] = None,
+              cls: Optional[type[XMLSchemaBase]] = None,
               path: Optional[str] = None,
               validation: str = 'strict',
-              namespaces: Optional[NamespacesType] = None,
+              namespaces: Optional[NsmapType] = None,
               use_defaults: bool = True,
               converter: Optional[ConverterType] = None,
               unordered: bool = False,
-              json_options: Optional[Dict[str, Any]] = None,
+              json_options: Optional[dict[str, Any]] = None,
               **kwargs: Any) -> EncodeType[ElementType]:
     """
     Deserialize JSON data to an XML Element.
@@ -538,112 +531,72 @@ class XmlDocument(XMLResource):
     :param namespaces: is an optional mapping from namespace prefix to URI.
     :param locations: resource location hints, that can be a dictionary or a \
     sequence of couples (namespace URI, resource URL).
-    :param base_url: the base URL for base :class:`xmlschema.XMLResource` initialization.
-    :param allow: the security mode for base :class:`xmlschema.XMLResource` initialization.
-    :param defuse: the defuse mode for base :class:`xmlschema.XMLResource` initialization.
-    :param timeout: the timeout for base :class:`xmlschema.XMLResource` initialization.
-    :param lazy: the lazy mode for base :class:`xmlschema.XMLResource` initialization.
     :param use_location_hints: for default, in case a schema instance has \
     to be built, uses also schema locations hints provided within XML data. \
-    Set this option to `False` to ignore these schema location hints.
+    set this option to `False` to ignore these schema location hints.
+    :param kwargs: other optional arguments for building :class:`XMLResource` or \
+    :class:`XMLSchema` instances provided as keyword arguments.
     """
-    schema: Optional[XMLSchemaBase] = None
-    _fallback_schema: Optional[XMLSchemaBase] = None
-    validation: str = 'skip'
-    namespaces: Optional[NamespacesType] = None
-    errors: Union[Tuple[()], List[XMLSchemaValidationError]] = ()
+    errors: Union[tuple[()], list[XMLSchemaValidationError]] = ()
+
+    # Additional arguments
+    schema: SchemaArgument = SchemaArgument()
+    _schema: SchemaType
+    validation: ValidationOption = ValidationOption(default='strict')
+    locations: LocationsOption = LocationsOption(default=None)
+    use_location_hints: BooleanOption = BooleanOption(default=True)
 
     def __init__(self, source: XMLSourceType,
-                 schema: Optional[Union[XMLSchemaBase, SchemaSourceType]] = None,
-                 cls: Optional[Type[XMLSchemaBase]] = None,
+                 schema: Optional[Union[XMLSchemaBase, SourceArgType]] = None,
+                 cls: Optional[type[XMLSchemaBase]] = None,
                  validation: str = 'strict',
-                 namespaces: Optional[NamespacesType] = None,
+                 namespaces: Optional[NsmapType] = None,
                  locations: Optional[LocationsType] = None,
-                 base_url: Optional[str] = None,
-                 allow: str = 'all',
-                 defuse: str = 'remote',
-                 timeout: int = 300,
-                 lazy: LazyType = False,
-                 use_location_hints: bool = True) -> None:
+                 use_location_hints: bool = True,
+                 **kwargs: Any) -> None:
+
+        super().__init__(source, **{k: kwargs[k] for k in kwargs if k in RESOURCE_KWARGS})
 
-        if cls is None:
-            cls = XMLSchema10
         self.validation = validation
-        self._namespaces = namespaces
-        super(XmlDocument, self).__init__(source, base_url, allow, defuse, timeout, lazy)
+        self.use_location_hints = use_location_hints
+        self._init_namespaces = get_namespace_map(namespaces)
+        self.namespaces = super().get_namespaces(namespaces, root_only=True)
+        self.schema = get_resource_schema(
+            resource=self,
+            schema=schema,
+            cls=cls,
+            validation=validation,
+            locations=locations,
+            use_location_hints=use_location_hints,
+            **{k: kwargs[k] for k in kwargs if k in SCHEMA_KWARGS}
+        )
 
-        if isinstance(schema, XMLSchemaBase) and self.namespace in schema.maps.namespaces:
-            self.schema = schema
-        elif schema is not None and not isinstance(schema, XMLSchemaBase):
-            self.schema = cls(
-                source=schema,
-                locations=locations,
-                base_url=base_url,
-                allow=allow,
-                defuse=defuse,
-                timeout=timeout,
-            )
-        else:
-            if use_location_hints:
-                try:
-                    schema_location, locations = fetch_schema_locations(
-                        self, locations=locations, base_url=base_url
-                    )
-                except ValueError:
-                    pass
-                else:
-                    self.schema = cls(
-                        source=schema_location,
-                        locations=locations,
-                        allow=allow,
-                        defuse=defuse,
-                        timeout=timeout,
-                    )
-
-            if self.schema is None:
-                if XSI_TYPE in self._root.attrib:
-                    self.schema = cls.meta_schema
-                elif validation != 'skip':
-                    msg = "cannot get a schema for XML data, provide a schema argument"
-                    raise XMLSchemaValueError(msg)
-                else:
-                    self._fallback_schema = get_dummy_schema(self.root.tag, cls)
-
-        if self.schema is None:
-            pass
-        elif validation == 'strict':
-            self.schema.validate(self, namespaces=self.namespaces)
+        if validation == 'strict':
+            self._schema.validate(self, namespaces=self.namespaces)
         elif validation == 'lax':
-            self.errors = [e for e in self.schema.iter_errors(self, namespaces=self.namespaces)]
+            self.errors = [e for e in self._schema.iter_errors(self, namespaces=self.namespaces)]
         elif validation != 'skip':
             raise XMLSchemaValueError("%r is not a validation mode" % validation)
 
-    def parse(self, source: XMLSourceType, lazy: LazyType = False) -> None:
-        super(XmlDocument, self).parse(source, lazy)
-        self.namespaces = self.get_namespaces()
+    def get_arguments(self) -> dict[str, Any]:
+        """Returns keyword arguments for rebuilding the XML document."""
+        kwargs = super().get_arguments()
+        kwargs.update(
+            validation=self.validation,
+            schema=self.schema,
+            namespaces=self._init_namespaces
+        )
+        return kwargs
 
-        if self.schema is None:
-            pass
-        elif self.validation == 'strict':
-            self.schema.validate(self, namespaces=self.namespaces)
-        elif self.validation == 'lax':
-            self.errors = [e for e in self.schema.iter_errors(self, namespaces=self.namespaces)]
-
-    def get_namespaces(self, namespaces: Optional[NamespacesType] = None,
-                       root_only: Optional[bool] = None) -> NamespacesType:
-        if not self._namespaces:
-            _namespaces = namespaces
-        elif not namespaces:
-            _namespaces = self._namespaces
-        else:
-            _namespaces = copy.copy(self._namespaces)
-            _namespaces.update(namespaces)
-
-        return super().get_namespaces(_namespaces, root_only)
+    def get_namespaces(self, namespaces: Optional[NsmapType] = None,
+                       root_only: bool = True, root_default: bool = False) -> dict[str, str]:
+        namespaces = get_namespace_map(namespaces)
+        update_namespaces(namespaces, self.namespaces.items(), root_declarations=True)
+        return super().get_namespaces(namespaces, root_only, root_default)
 
     def getroot(self) -> ElementType:
         """Get the root element of the XML document."""
-        return self._root
+        return self.root
 
     def get_etree_document(self) -> Any:
         """
@@ -656,10 +609,10 @@ class XmlDocument(XMLResource):
             raise XMLResourceError(
                 "cannot create an ElementTree instance from a lazy XML resource"
             )
-        elif hasattr(self._root, 'nsmap'):
-            return self._root.getroottree()  # type: ignore[attr-defined]
+        elif hasattr(self.root, 'getroottree'):
+            return self.root.getroottree()
         else:
-            return ElementTree.ElementTree(self._root)
+            return ElementTree.ElementTree(self.root)
 
     def decode(self, **kwargs: Any) -> DecodeType[Any]:
         """
@@ -672,15 +625,11 @@ class XmlDocument(XMLResource):
         if 'namespaces' not in kwargs:
             kwargs['namespaces'] = self.namespaces
 
-        schema = self.schema or self._fallback_schema
-        if schema is None:
-            return None
-
-        obj = schema.to_dict(self, **kwargs)
+        obj = self._schema.to_dict(self, **kwargs)
         return obj[0] if isinstance(obj, tuple) else obj
 
     def to_json(self, fp: Optional[IO[str]] = None,
-                json_options: Optional[Dict[str, Any]] = None,
+                json_options: Optional[dict[str, Any]] = None,
                 **kwargs: Any) -> JsonDecodeType:
         """
         Converts loaded XML data to a JSON string or file.
@@ -699,18 +648,13 @@ class XmlDocument(XMLResource):
         if 'decimal_type' not in kwargs:
             kwargs['decimal_type'] = float
 
-        errors: List[XMLSchemaValidationError] = []
+        errors: list[XMLSchemaValidationError] = []
 
         if path is None and self._lazy and 'cls' not in json_options:
             json_options['cls'] = get_lazy_json_encoder(errors)
             kwargs['lazy_decode'] = True
 
-        schema = self.schema or self._fallback_schema
-        if schema is not None:
-            obj = schema.decode(self, path=path, **kwargs)
-        else:
-            obj = None
-
+        obj = self._schema.decode(self, path=path, **kwargs)
         if isinstance(obj, tuple):
             if fp is not None:
                 json.dump(obj[0], fp, **json_options)
@@ -735,7 +679,7 @@ class XmlDocument(XMLResource):
         if self._lazy:
             raise XMLResourceError("cannot serialize a lazy XML resource")
 
-        kwargs: Dict[str, Any] = {
+        kwargs: dict[str, Any] = {
             'xml_declaration': xml_declaration,
             'encoding': encoding,
             'method': method,
@@ -743,40 +687,38 @@ class XmlDocument(XMLResource):
         if not default_namespace:
             kwargs['namespaces'] = self.namespaces
         else:
-            namespaces: Optional[Dict[Optional[str], str]]
-            if self.namespaces is None:
-                namespaces = {}
-            else:
-                namespaces = {k: v for k, v in self.namespaces.items()}
+            namespaces: Optional[dict[Optional[str], str]]
+            namespaces = {k: v for k, v in self.namespaces.items()}
 
-            if hasattr(self._root, 'nsmap'):
+            if hasattr(self.root, 'nsmap'):
                 # noinspection PyTypeChecker
                 namespaces[None] = default_namespace
             else:
                 namespaces[''] = default_namespace
             kwargs['namespaces'] = namespaces
 
-        _string = etree_tostring(self._root, **kwargs)
+        _string = etree_tostring(self.root, **kwargs)
 
-        if isinstance(file, str):
-            if isinstance(_string, str):
-                with open(file, 'w', encoding='utf-8') as fp:
-                    fp.write(_string)
-            else:
-                with open(file, 'wb') as _fp:
-                    _fp.write(_string)
+        match file:
+            case str():
+                if isinstance(_string, str):
+                    with open(file, 'w', encoding='utf-8') as fp:
+                        fp.write(_string)
+                else:
+                    with open(file, 'wb') as _fp:
+                        _fp.write(_string)
 
-        elif isinstance(file, TextIOBase):
-            if isinstance(_string, bytes):
-                file.write(_string.decode('utf-8'))
-            else:
-                file.write(_string)
+            case TextIOBase():
+                if isinstance(_string, bytes):
+                    file.write(_string.decode('utf-8'))
+                else:
+                    file.write(_string)
 
-        elif isinstance(file, IOBase):
-            if isinstance(_string, str):
-                file.write(_string.encode('utf-8'))
-            else:
-                file.write(_string)
-        else:
-            msg = "unexpected type %r for 'file' argument"
-            raise XMLSchemaTypeError(msg % type(file))
+            case IOBase():
+                if isinstance(_string, str):
+                    file.write(_string.encode('utf-8'))
+                else:
+                    file.write(_string)
+            case _:
+                msg = "unexpected type %r for 'file' argument"
+                raise XMLSchemaTypeError(msg % type(file))
